@@ -6,6 +6,8 @@ simpler to use these than writing them as a string into the protocol-file.
 
 import numpy as np
 from bluesky import plan_stubs as bps
+from bluesky.preprocessors import contingency_wrapper, rewindable_wrapper
+from bluesky.utils import all_safe_rewind, short_uid
 
 from ophyd import SignalRO, Device
 
@@ -20,12 +22,15 @@ from PySide6.QtWidgets import (
     QProgressBar,
     QPushButton,
     QComboBox,
+    QTextEdit,
+    QCheckBox,
 )
-from PySide6.QtCore import Signal, Qt
-from PySide6.QtGui import QFont
+from PySide6.QtCore import Signal, Qt, QDateTime
+from PySide6.QtGui import QFont, QIcon, QKeyEvent
 
 from nomad_camels.ui_widgets.add_remove_table import AddRemoveTable
 from nomad_camels.ui_widgets.channels_check_table import Channels_Check_Table
+from nomad_camels import graphics
 
 from suitcase.nomad_camels_hdf5 import Serializer, export
 
@@ -34,6 +39,8 @@ import os
 import sys
 import glob
 
+from importlib import resources
+
 from nomad_camels.bluesky_handling.loop_step_functions_api_call import (
     execute_camels_api_call,
     execute_generic_api_call,
@@ -44,25 +51,42 @@ from nomad_camels.bluesky_handling.loop_step_functions_api_call import (
 
 def get_newest_file(directory):
     """
-    Return the newest file in the given directory.
+    Return the newest '.nxs' file in the specified directory, if one exists.
+
+    This function searches the given directory for files ending with the '.nxs'
+    extension and returns the one with the most recent modification time.
+
+    If the input 'directory' is a file, its parent directory is used for the search.
+    If no '.nxs' files are found, the function returns None.
 
     Parameters
     ----------
     directory : str
-        The directory where the newest file should be found.
+        The directory to search for '.nxs' files. If a file path is provided,
+        the search is performed in the file's parent directory.
 
     Returns
     -------
-    newest_file : str
-        The path to the newest file in the directory.
+    str or None
+        The full path to the newest '.nxs' file found in the directory, or None
+        if no such file exists.
     """
-    # If the given directory is a file, take the directory of the file
+    # If the given path is a file, use its directory
     if os.path.isfile(directory):
         directory = os.path.dirname(directory)
-    # List all files in the directory
-    files = glob.glob(os.path.join(directory, "*"))
-    # Find the newest file
-    newest_file = max(files, key=os.path.getmtime)
+
+    # Define the search pattern for '.nxs' files
+    pattern = os.path.join(directory, "*.nxs")
+
+    # Retrieve a list of all '.nxs' files in the directory
+    nxs_files = glob.glob(pattern)
+
+    # If no '.nxs' files are found, return None
+    if not nxs_files:
+        return None
+
+    # Return the newest file based on its modification time
+    newest_file = max(nxs_files, key=os.path.getmtime)
     return newest_file
 
 
@@ -74,6 +98,7 @@ def export_function(
     export_csv=False,
     export_json=False,
     plot_data=None,
+    do_nexus_output=True,
 ):
     """
     Export the given runs to the given path.
@@ -109,7 +134,14 @@ def export_function(
             runs = [runs]
         for run in runs:
             docs = run.documents(fill="yes")
-            export(docs, path, fname, new_file_each, plot_data=plot_data)
+            export(
+                docs,
+                path,
+                fname,
+                new_file_each,
+                plot_data=plot_data,
+                do_nexus_output=do_nexus_output,
+            )
     if export_csv or export_json:
         from nomad_camels.utility.databroker_export import export_h5_to_csv_json
 
@@ -117,7 +149,9 @@ def export_function(
         export_h5_to_csv_json(file, export_data=export_csv, export_metadata=export_json)
 
 
-def saving_function(name, start_doc, path, new_file_each=True, plot_data=None):
+def saving_function(
+    name, start_doc, path, new_file_each=True, plot_data=None, do_nexus_output=True
+):
     """
     Create a Serializer and return it.
 
@@ -152,7 +186,13 @@ def saving_function(name, start_doc, path, new_file_each=True, plot_data=None):
         fname = os.path.basename(path)
         path = os.path.dirname(path)
     return [
-        Serializer(path, fname, new_file_each=new_file_each, plot_data=plot_data)
+        Serializer(
+            path,
+            fname,
+            new_file_each=new_file_each,
+            plot_data=plot_data,
+            do_nexus_output=do_nexus_output,
+        )
     ], []
 
 
@@ -173,7 +213,7 @@ def trigger_multi(devices, grp=None):
             yield from bps.trigger(obj, group=grp)
 
 
-def read_wo_trigger(devices, grp=None, stream="primary"):
+def read_wo_trigger(devices, grp=None, stream="primary", skip_on_exception=None):
     """
     Used if not reading by trigger_and_read, but splitting both. This function only reads, without triggering.
 
@@ -192,18 +232,48 @@ def read_wo_trigger(devices, grp=None, stream="primary"):
     ret : dict
         The readings of the devices.
     """
+    skip_on_exception = skip_on_exception or [False] * len(devices)
+    for i, obj in enumerate(devices):
+        if skip_on_exception[i]:
+            obj.__read_w_except__ = True
+        else:
+            obj.__read_w_except__ = False
     if grp is not None:
         yield from bps.wait(grp)
     yield from bps.create(stream)
-    ret = {}  # collect and return readings to give plan access to them
-    for obj in devices:
-        reading = yield from bps.read(obj)
-        if reading is not None:
-            ret.update(reading)
-    yield from bps.save()
+
+    def read_plan():
+        ret = {}  # collect and return readings to give plan access to them
+        for obj in devices:
+            reading = yield from bps.read(obj)
+            if reading is not None:
+                ret.update(reading)
+        return ret
+
+    def standard_path():
+        yield from bps.save()
+
+    def exception_path(ex):
+        yield from bps.drop()
+        raise ex
+
+    ret = yield from contingency_wrapper(
+        read_plan(), except_plan=exception_path, else_plan=standard_path
+    )
     return ret
 
 
+def trigger_and_read(devices, name="primary", skip_on_exception=None):
+    rewindable = all_safe_rewind(devices)
+
+    def inner_trigger_read():
+        grp = short_uid("trigger")
+        yield from trigger_multi(devices, grp)
+        return (yield from read_wo_trigger(devices, grp, name, skip_on_exception))
+
+    return (yield from rewindable_wrapper(inner_trigger_read(), rewindable))
+
+
 def simplify_configs_dict(configs):
     """
     Returns a simplified version of the given dictionary `configs` by returning confs[key] = configs[key]['value'].
@@ -257,8 +327,6 @@ def get_fit_results(fits, namespace, yielding=False, stream="primary"):
                 [fit.read_ready], name=f"{stream}_fits_readying_{name}"
             )
             yield from fit.update_fit()
-            # yield from bps.trigger_and_read(fit.ophyd_fit.used_comps,
-            #                                 name=f'{stream}_fits_{name}')
         if not fit.result:
             continue
         for param in fit.params:
@@ -740,19 +808,6 @@ class Value_Box(QDialog):
         self.helper.executor.connect(self.start_execution)
         self.done_flag = False
 
-        channels_label = QLabel("Channels")
-        font = QFont()
-        font.setBold(True)
-        channels_label.setStyleSheet("font-size: 9pt")
-        channels_label.setFont(font)
-        layout.addWidget(channels_label, 1, 0, 1, 2)
-        variables_label = QLabel("Variables")
-        font = QFont()
-        font.setBold(True)
-        variables_label.setStyleSheet("font-size: 9pt")
-        variables_label.setFont(font)
-        layout.addWidget(variables_label, 1, 2, 1, 2)
-
         channels = channels or []
         n_channels = len(channels)
         self.channel_boxes = []
@@ -779,6 +834,19 @@ class Value_Box(QDialog):
             layout.addWidget(variable_label, 2 + i, 2)
             layout.addWidget(variable_box, 2 + i, 3)
 
+        font = QFont()
+        font.setBold(True)
+        if channels:
+            channels_label = QLabel("Channels")
+            channels_label.setStyleSheet("font-size: 9pt")
+            channels_label.setFont(font)
+            layout.addWidget(channels_label, 1, 0, 1, 2)
+        if variables:
+            variables_label = QLabel("Variables")
+            variables_label.setStyleSheet("font-size: 9pt")
+            variables_label.setFont(font)
+            layout.addWidget(variables_label, 1, 2, 1, 2)
+
         devs = devs or {}
         self.channel_devs = {}
         for dev, val in devs.items():
@@ -868,8 +936,14 @@ class Value_Box(QDialog):
         Sets `self.done_flag` to True, allowing the protocol to go on before
         rejecting the dialog.
         """
-        self.done_flag = True
-        return super().reject()
+        msg = QMessageBox(
+            icon=QMessageBox.Warning,
+            text="If you cancel, no values will be set.\nDo you want to cancel?",
+        )
+        msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
+        if msg.exec() == QMessageBox.Yes:
+            self.done_flag = True
+            return super().reject()
 
 
 def get_channels(dev):
@@ -1087,24 +1161,35 @@ class Value_Setter(QWidget):
     hide_signal = Signal()
 
     def __init__(self, parent=None):
+        import datetime as dt
+
         super().__init__(parent)
-        self.start_time = 0
-        self.end_time = 0
+        self.start_time = dt.datetime.now()
+        self.end_time = dt.datetime.now()
         self.timer = 0
-        self.wait_time = 0
+        self.wait_time = 1
+
+    def set_start_time(self, start_time):
+        """Set the start time for the waiting bar."""
+        self.start_time = start_time
+        self.set_wait_time(self.wait_time)
+        self.update_timer()
 
     def set_wait_time(self, wait_time):
         """Set the wait time for the waiting bar."""
         import datetime as dt
 
         self.wait_time = dt.timedelta(seconds=wait_time).total_seconds()
+        self.update_timer()
 
     def update_timer(self):
         """Update the timer of the waiting bar."""
         import datetime as dt
 
         self.timer = (dt.datetime.now() - self.start_time).total_seconds()
-        self.set_signal.emit(self.timer / self.wait_time * 100)
+        self.set_signal.emit(
+            self.timer / (self.wait_time or 1) * 100
+        )  # Prevent division by zero
 
 
 class Waiting_Bar(QWidget):
@@ -1184,6 +1269,70 @@ class Waiting_Bar(QWidget):
         if self.with_timer:
             import datetime as dt
 
-            self.setter.start_time = dt.datetime.now()
+            self.setter.set_start_time(dt.datetime.now())
 
         self.show()
+
+
+class TimestampTextEdit(QTextEdit):
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        # Flag to track if the first keystroke has been handled.
+        self.first_key_pressed = False
+
+    def insertTimestamp(self):
+        """
+        Inserts the current timestamp followed by a space at the current cursor position.
+        The timestamp is in ISO 8601 format (e.g. "2025-02-05T14:30:00").
+        """
+        timestamp = QDateTime.currentDateTime().toString(Qt.ISODate)
+        self.insertPlainText(timestamp + " ")
+
+    def keyPressEvent(self, event: QKeyEvent):
+        # If this is the very first key press in the widget,
+        # prepend a timestamp.
+        if not self.first_key_pressed:
+            self.insertTimestamp()
+            self.first_key_pressed = True
+
+        # Process the key event normally.
+        super().keyPressEvent(event)
+
+        # If the key pressed is Return or Enter, then insert a new timestamp
+        # on the new line.
+        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
+            self.insertTimestamp()
+
+
+class Commenting_Box(QWidget):
+    """A widget to add comments to the protocol."""
+
+    closing = Signal()
+
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        layout = QGridLayout()
+        self.comment_box = TimestampTextEdit()
+        self.finished_box = QCheckBox("commenting finished")
+        self.setWindowTitle("Live Measurement Comments - NOMAD CAMELS")
+        self.setWindowIcon(QIcon(str(resources.files(graphics) / "camels_icon.png")))
+        layout.addWidget(self.comment_box, 0, 0)
+        layout.addWidget(self.finished_box, 1, 0)
+        self.setLayout(layout)
+        self.adjustSize()
+        self._is_finished = False
+        self.finished_box.clicked.connect(self.finish_comment)
+        self.setWindowFlags(self.windowFlags() & ~Qt.WindowCloseButtonHint)
+        self.show()
+
+    def finish_comment(self):
+        """Sets the flag `_is_finished` to True."""
+        self._is_finished = True
+
+    def get_metadata(self):
+        """Get the text of the comment."""
+        return {"measurement_comments": self.comment_box.toPlainText()}
+
+    def closeEvent(self, event):
+        self.closing.emit()
+        return super().closeEvent(event)
