@@ -55,7 +55,8 @@ standard_string = "import numpy as np\n"
 standard_string += "import importlib\n"
 standard_string += "import bluesky\n"
 standard_string += "import ophyd\n"
-standard_string += "from bluesky import RunEngine\n"
+standard_string += "import requests\n"
+standard_string += "from nomad_camels.bluesky_handling.run_engine_overwrite import RunEngineOverwrite\n"
 standard_string += "from bluesky.callbacks.best_effort import BestEffortCallback\n"
 standard_string += "import bluesky.plan_stubs as bps\n"
 standard_string += "import databroker\n"
@@ -63,7 +64,6 @@ standard_string += "from PySide6.QtWidgets import QApplication, QMessageBox\n"
 standard_string += "from PySide6.QtCore import QCoreApplication, QThread\n"
 standard_string += "import datetime\n"
 standard_string += "import subprocess\n"
-standard_string += "from nomad_camels.main_classes import plot_pyqtgraph, list_plot\n"
 standard_string += "from nomad_camels.utility import theme_changing\n"
 standard_string += (
     "from nomad_camels.bluesky_handling.evaluation_helper import Evaluator\n"
@@ -91,11 +91,12 @@ standard_run_string += "\tdarkmode, theme = dark, used_theme\n"
 # and also the if branch, whether it is the main script to execute everything
 # without importing
 standard_start_string = "\n\n\ndef main():\n"
-standard_start_string += "\tRE = RunEngine()\n"
+standard_start_string += "\tRE = RunEngineOverwrite()\n"
 standard_start_string += "\tbec = BestEffortCallback()\n"
 standard_start_string += "\tRE.subscribe(bec)\n"
 standard_start_string2 = "\t\tplot_etc = create_plots(RE)\n"
 standard_start_string2 += "\t\tadditional_step_data = steps_add_main(RE, devs)\n"
+standard_start_string2 += "\t\tcreate_live_windows()\n"
 standard_start_string2 += (
     "\t\trun_protocol_main(RE=RE, catalog=catalog, devices=devs, md=md)\n"
 )
@@ -216,19 +217,19 @@ def build_protocol(
     variable_string = "\nnamespace = {}\n"
     variable_string += "all_fits = {}\n"
     variable_string += "plots = []\n"
-    variable_string += "plot_data = []\n"
+    variable_string += "plots_plotly = []\n"
     variable_string += "boxes = {}\n"
+    variable_string += "live_windows = []\n"
     variable_string += "app = None\n"
     variable_string += f'save_path = "{save_path}"\n'
     variable_string += f'session_name = "{protocol.session_name}"\n'
     variable_string += f"export_to_csv = {protocol.export_csv}\n"
-    variable_string += f"export_to_json={protocol.export_json}\n"
+    variable_string += f"export_to_json = {protocol.export_json}\n"
     if "new_file_each_run" in variables_handling.preferences:
-        variable_string += (
-            f'new_file_each_run={variables_handling.preferences["new_file_each_run"]}\n'
-        )
+        variable_string += f'new_file_each_run = {variables_handling.preferences["new_file_each_run"]}\n'
     else:
-        variable_string += f"new_file_each_run=False\n"
+        variable_string += f"new_file_each_run = False\n"
+    variable_string += f"do_nexus_output = {protocol.use_nexus}\n"
     additional_string_devices = ""
     final_string = "\t\tfor name, device in devs.items():\n"
     final_string += '\t\t\tif hasattr(device, "finalize_steps") and callable(device.finalize_steps):\n'
@@ -251,6 +252,7 @@ def build_protocol(
         variable_string += f"{var} = {val}\n"
         variable_string += f'namespace["{var}"] = {var}\n'
     variable_string += f'\n{protocol.name}_variable_signal = variable_reading.Variable_Signal(name="{protocol.name}_variable_signal", variables_dict=namespace)\n'
+    variable_string += 'eva = Evaluator(namespace=namespace)\n'
     # this handles all the used devices
     for dev in protocol.get_used_devices():
         device = variables_handling.devices[dev]
@@ -326,14 +328,7 @@ def build_protocol(
 
     # finishing up the device initialization
     devices_string += '\t\tprint("devices connected")\n'
-    devices_string += f'\t\tmd = {{"devices": device_config, "description": "{repr(protocol.description)}"}}\n'
-
-    # if using special nexus-format, some restructuring of metadata is done
-    if protocol.use_nexus:
-        md_dict = {}
-        for i, name in enumerate(protocol.metadata["Name"]):
-            md_dict[name] = protocol.metadata["Value"][i]
-        devices_string += f"\t\tmd.update({md_dict})\n"
+    devices_string += f'\t\tmd = {{"devices": device_config}}\n'
 
     # the plots are created
     plot_string, plotting = plot_creator(protocol.plots, multi_stream=True)
@@ -354,6 +349,7 @@ def build_protocol(
     protocol_string += protocol.get_plan_string()
     protocol_string += plot_string
     protocol_string += protocol.get_add_main_string()
+    protocol_string += protocol.get_live_interaction_string()
     protocol_string += standard_run_string
 
     # setting up the progress bar
@@ -366,6 +362,11 @@ def build_protocol(
     protocol_string += user_sample_string(userdata, sampledata)
     protocol_string += f'\tmd["session_name"] = session_name\n'
     protocol_string += f'\tmd["protocol_overview"] = """{protocol.get_short_string().encode("unicode_escape").decode()}"""\n'
+    protocol_string += f'\tmd["description"] = {repr(protocol.description)}\n'
+    protocol_string += f'\tmd["measurement_tags"] = {protocol.tags}\n'
+    protocol_string += (
+        f'\tmd["measurement_description"] = {repr(protocol.measurement_description)}\n'
+    )
 
     protocol_string += f"\ttry:\n"
     protocol_string += f'\t\twith open("{cprot_path}", "r", encoding="utf-8") as f:\n'
@@ -378,7 +379,7 @@ def build_protocol(
     protocol_string += '\t\tmd["python_script"] = f.read()\n'
     protocol_string += read_device_drivers_string
     protocol_string += '\tmd["variables"] = namespace\n'
-    protocol_string += '\tmd["plot_data"] = plot_data\n'
+    # protocol_string += '\tmd["plot_data"] = plot_data\n'
 
     # adding uid to RunEngine, calling the plan
     protocol_string += '\tsubscription_uid = RE.subscribe(uid_collector, "start")\n'
@@ -386,25 +387,20 @@ def build_protocol(
     protocol_string += f"\t\tRE({protocol.name}_plan(devs, md=md, runEngine=RE))\n"
     protocol_string += "\tfinally:\n"
     protocol_string += "\t\tRE.unsubscribe(subscription_uid)\n"
+    protocol_string += "\t\tfor window in live_windows:\n"
+    protocol_string += "\t\t\twindow.close()\n"
 
     # wait for RunEngine to finish, then save the data
     save_string = "\t\tif uids:\n"
     save_string += "\t\t\truns = catalog[tuple(uids)]\n"
-    if protocol.use_nexus:
-        nexus_dict = protocol.get_nexus_paths()
-        nexus_dict.update(standard_nexus_dict)
-        save_string += "\t\t\tdata = broker_to_dict(runs)\n"
-        save_string += f"\t\t\tnexus_mapper = {nexus_dict}\n\n"
-        # TODO finish this
-    else:
-        save_string += f"\t\t\thelper_functions.export_function(runs, save_path, {not protocol.h5_during_run}, new_file_each=new_file_each_run, plot_data=plots)\n"
-        # save_string += (
-        #     "\t\t\tbroker_to_NX(runs, save_path, plots,"
-        #     "session_name=session_name,"
-        #     "export_to_csv=export_to_csv,"
-        #     "export_to_json=export_to_json,"
-        #     "new_file_each_run=new_file_each_run)\n\n"
-        # )
+    save_string += f"\t\t\thelper_functions.export_function(runs, save_path, {not protocol.h5_during_run}, new_file_each=new_file_each_run, plot_data=plots, do_nexus_output=do_nexus_output)\n"
+    # save_string += (
+    #     "\t\t\tbroker_to_NX(runs, save_path, plots,"
+    #     "session_name=session_name,"
+    #     "export_to_csv=export_to_csv,"
+    #     "export_to_json=export_to_json,"
+    #     "new_file_each_run=new_file_each_run)\n\n"
+    # )
     if protocol.h5_during_run:
         save_string += "\t\tRE.unsubscribe(subscription_rr)\n"
 
@@ -440,7 +436,7 @@ def build_protocol(
     protocol_string += "\tdevs = {}\n\tdevice_config = {}\n\ttry:\n"
     protocol_string += devices_string
     if protocol.h5_during_run:
-        protocol_string += "\t\trr = RunRouter([lambda x, y: helper_functions.saving_function(x, y, save_path, new_file_each_run, plots)])\n"
+        protocol_string += "\t\trr = RunRouter([lambda x, y: helper_functions.saving_function(x, y, save_path, new_file_each_run, plots, do_nexus_output)])\n"
         protocol_string += "\t\tsubscription_rr = RE.subscribe(rr)\n"
     protocol_string += standard_start_string2
     protocol_string += standard_final_string
@@ -449,8 +445,6 @@ def build_protocol(
     protocol_string += standard_start_string3
     protocol_string = protocol_string.expandtabs(4)
 
-
-
     # the string is written to the file
     if not os.path.isdir(os.path.dirname(file_path)):
         os.makedirs(os.path.dirname(file_path))
@@ -514,7 +508,7 @@ def sub_protocol_string(
     stream = prot_name
     if data_output == "main stream":
         stream = "primary"
-    protocol_string += f'{tabs}yield from {prot_name}_mod.{prot_name}_plan_inner(devs, {prot_name}_eva, "{stream}", runEngine)\n'
+    protocol_string += f'{tabs}yield from {prot_name}_mod.{prot_name}_plan_inner(devs, "{stream}", runEngine)\n'
     for i, var in enumerate(variables_out["Variable"]):
         protocol_string += f'{tabs}namespace["{variables_out["Write to name"][i]}"] = {prot_name}_mod.namespace["{var}"]\n'
     protocol_string += f"{tabs}runEngine.unsubscribe(sub_eva_{prot_name})\n"
