@@ -1,7 +1,13 @@
 from PySide6.QtWidgets import QDialog, QStyleFactory, QMessageBox, QApplication
 from PySide6.QtCore import Qt, QCoreApplication
 from PySide6.QtGui import QKeyEvent
-import qt_material
+
+try:
+    import qt_material
+
+    QT_MATERIAL = True
+except ImportError:
+    QT_MATERIAL = False
 
 from nomad_camels.gui.settings_window import Ui_settings_window
 from nomad_camels.utility import load_save_functions
@@ -35,9 +41,10 @@ class Settings_Window(Ui_settings_window, QDialog):
             app = QCoreApplication.instance()
             self.comboBox_theme.setCurrentText(app.style().objectName())
         material_themes = []
-        for t in qt_material.list_themes():
-            if t.startswith("light_"):
-                material_themes.append(t[6:-4])
+        if QT_MATERIAL:
+            for t in qt_material.list_themes():
+                if t.startswith("light_"):
+                    material_themes.append(t[6:-4])
         self.comboBox_material_theme.addItems(material_themes)
         if (
             "material_theme" in settings
@@ -132,6 +139,9 @@ class Settings_Window(Ui_settings_window, QDialog):
             self.checkBox_play_camel_on_error.setChecked(
                 standard_pref["play_camel_on_error"]
             )
+        self.checkBox_finished_sound.setChecked(
+            settings.get("finished_sound", standard_pref["finished_sound"])
+        )
 
         for level in log_levels:
             self.comboBox_log_level.addItem(level)
@@ -195,6 +205,10 @@ class Settings_Window(Ui_settings_window, QDialog):
                 self.pushButton_delete_Api_keys.setEnabled(False)
         if "API_port" in settings:
             self.lineEdit_api_port.setText(settings["API_port"])
+        self.pushButton_API_docu.clicked.connect(self.open_api_docu)
+        self.pushButton_API_docu.setToolTip(
+            "If the page does not load, accept your settings and try again. The API server might not be started yet."
+        )
 
     def autosave_run_change(self):
         on = self.checkBox_autosave_run.isChecked()
@@ -280,6 +294,7 @@ class Settings_Window(Ui_settings_window, QDialog):
             "databroker_catalog_name": self.lineEdit_catalog_name.text(),
             "number_databroker_files": self.spinBox_n_databroker_files.value(),
             "play_camel_on_error": self.checkBox_play_camel_on_error.isChecked(),
+            "finished_sound": self.checkBox_finished_sound.isChecked(),
             "log_level": self.comboBox_log_level.currentText(),
             "logfile_size": self.spinBox_logfile_size.value(),
             "logfile_backups": self.spinBox_logfile_number.value(),
@@ -374,14 +389,108 @@ class Settings_Window(Ui_settings_window, QDialog):
 
     def change_enable_API(self):
         if self.checkBox_enable_Api.isChecked():
-            self.pushButton_generate_Api_key.setEnabled(True)
-            self.pushButton_delete_Api_keys.setEnabled(True)
+            if self.check_if_api_modules_are_available():
+                self.pushButton_generate_Api_key.setEnabled(True)
+                self.pushButton_delete_Api_keys.setEnabled(True)
+                self.pushButton_API_docu.setEnabled(True)
+            else:
+                self.checkBox_enable_Api.setChecked(False)
         else:
             self.pushButton_generate_Api_key.setEnabled(False)
             self.pushButton_copy_Api_key_clipboard.setEnabled(False)
             self.pushButton_delete_Api_keys.setEnabled(False)
+            self.pushButton_API_docu.setEnabled(False)
             self.pushButton_copy_Api_key_clipboard.setStyleSheet("")
 
+    def open_api_docu(self):
+        from nomad_camels.utility import variables_handling
+
+        link = f"http://127.0.0.1:{self.lineEdit_api_port.text()}/docs"
+        variables_handling.open_link(link)
+
+    def check_if_api_modules_are_available(self):
+        # Function to check if a module is installed.
+        # List the required modules.
+        required_modules = ["fastapi", "uvicorn", "httpx"]
+
+        # Determine which modules are missing.
+        missing_modules = [
+            mod for mod in required_modules if not is_module_available(mod)
+        ]
+
+        if missing_modules:
+            # Create the message to warn the user.
+            msg = (
+                f"The following modules are required: {', '.join(missing_modules)}.\n\n"
+                "These modules may take a few seconds to install. You only need to do this once.\n"
+                "Do you want to install them now?"
+            )
+
+            # Show a question message box.
+            reply_update_modules = QMessageBox.question(
+                None,
+                "Install Required Modules",
+                msg,
+                QMessageBox.Yes | QMessageBox.No,
+                QMessageBox.No,
+            )
+
+            if reply_update_modules == QMessageBox.Yes:
+                try:
+                    import sys
+                    import subprocess
+
+                    # Build the pip install command.
+                    try:
+                        command = [
+                            sys.executable,
+                            "-m",
+                            "pip",
+                            "install",
+                            "nomad-camels[api]",
+                        ]
+                        subprocess.check_call(command)
+                        missing_modules = [
+                            mod
+                            for mod in required_modules
+                            if not is_module_available(mod)
+                        ]
+                        if missing_modules:
+                            raise Exception("Failed to install nomad-camels[api]")
+                        return True
+
+                    except Exception as e:
+                        print(e)
+                        command = [
+                            sys.executable,
+                            "-m",
+                            "pip",
+                            "install",
+                        ] + missing_modules
+                    # Optionally, you might show another popup or a console message indicating progress.
+                    subprocess.check_call(command)
+                    QMessageBox.information(
+                        None,
+                        "Installation Complete",
+                        "The required modules have been installed.\nYou can now start the API server and use the API.",
+                    )
+                    return True
+                except Exception as e:
+                    QMessageBox.critical(
+                        None,
+                        "Installation Failed",
+                        f"An error occurred during installation:\n{str(e)}",
+                    )
+                    return False
+                # Exit the application (or you could try to continue, if that makes sense in your context).
+            else:
+                QMessageBox.warning(
+                    None,
+                    "Modules Missing",
+                    "You can not use the API without the required modules!",
+                )
+                return False
+
 
 def hash_api_key(api_key):
     return hashlib.sha256(api_key.encode()).hexdigest()
@@ -393,3 +502,11 @@ def store_api_key(api_key, conn):
     c = conn.cursor()
     c.execute("INSERT INTO api_keys (key) VALUES (?)", (hashed_key,))
     conn.commit()
+
+
+def is_module_available(module_name):
+    try:
+        __import__(module_name)
+        return True
+    except ImportError:
+        return False
