@@ -101,6 +101,92 @@ def activate_dark_mode():
     colors = dark_mode_colors
 
 
+class ListDeque_skip:
+    def __init__(self, iterable=None, maxlen=None, skip_n_points=0):
+        if iterable is None:
+            iterable = []
+        self.maxlen = maxlen
+        self.skip_n_points = skip_n_points
+        self.counter_value = 0
+        if (
+            maxlen is None
+            or maxlen == np.inf
+            or (isinstance(maxlen, str) and maxlen.lower() in ["none", "inf", "np.inf"])
+        ):
+            self.data = list(iterable)
+        else:
+            if not isinstance(maxlen, int):
+                maxlen = int(maxlen)
+            self.data = deque(iterable, maxlen=maxlen)
+
+    def append(self, item):
+        self.counter_value += 1
+        if self.skip_n_points <= 0 or self.counter_value % self.skip_n_points == 0:
+            self.data.append(item)
+
+    def extend(self, iterable):
+        self.data.extend(iterable)
+
+    def __getitem__(self, index):
+        return self.data[index]
+
+    def __setitem__(self, index, value):
+        self.data[index] = value
+
+    def __delitem__(self, index):
+        del self.data[index]
+
+    def __len__(self):
+        return len(self.data)
+
+    def __iter__(self):
+        return iter(self.data)
+
+    def __repr__(self):
+        return repr(self.data)
+
+    def clear(self):
+        self.data.clear()
+
+    def pop(self):
+        return self.data.pop()
+
+    def popleft(self):
+        if isinstance(self.data, deque):
+            return self.data.popleft()
+        else:
+            raise AttributeError("'list' object has no attribute 'popleft'")
+
+    def appendleft(self, item):
+        if isinstance(self.data, deque):
+            self.data.appendleft(item)
+        else:
+            raise AttributeError("'list' object has no attribute 'appendleft'")
+
+    def extendleft(self, iterable):
+        if isinstance(self.data, deque):
+            self.data.extendleft(iterable)
+        else:
+            raise AttributeError("'list' object has no attribute 'extendleft'")
+
+    def rotate(self, n=1):
+        if isinstance(self.data, deque):
+            self.data.rotate(n)
+        else:
+            raise AttributeError("'list' object has no attribute 'rotate'")
+
+    def change_maxlen(self, maxlen):
+        self.maxlen = maxlen
+        if maxlen is None or maxlen == np.inf:
+            self.data = list(self.data)
+        else:
+            self.data = deque(self.data, maxlen=maxlen)
+
+    def change_skip_n_points(self, skip_n_points):
+        self.skip_n_points = skip_n_points
+        self.counter_value = 0
+
+
 class PlotWidget(QWidget):
     """Class for creating a plot widget.
 
@@ -145,6 +231,14 @@ class PlotWidget(QWidget):
         Whether to use a logarithmic y-axis, by default False
     logY2 : bool
         Whether to use a logarithmic y-axis for the right axis, by default False
+    top_left_x : int, optional
+        The x-coordinate of the top left corner of the plot widget, by default None
+    top_left_y : int, optional
+        The y-coordinate of the top left corner of the plot widget, by default None
+    plot_width : int, optional
+        The width of the plot widget, by default None
+    plot_height : int, optional
+        The height of the plot widget, by default None
     maxlen : int
         The maximum number of data points to show, by default np.inf
     use_bluesky : bool
@@ -178,10 +272,18 @@ class PlotWidget(QWidget):
         logX=False,
         logY=False,
         logY2=False,
+        manual_plot_position=False,
+        top_left_x="",
+        top_left_y="",
+        plot_width="",
+        plot_height="",
         maxlen=np.inf,
         use_bluesky=True,
         labels=(),
         first_hidden=None,
+        show_in_browser=False,
+        web_port=None,
+        evaluator=None,
         **kwargs,
     ):
         super().__init__(parent=parent)
@@ -190,7 +292,7 @@ class PlotWidget(QWidget):
         self.y_names = y_names or y_axes.keys()
         self.stream_name = stream_name
         self.fits = fits or []
-        self.eva = Evaluator(namespace=namespace)
+        self.eva = evaluator
         self.liveFits = []
         self.liveFitPlots = []
         self.ax2_viewbox = None
@@ -258,6 +360,8 @@ class PlotWidget(QWidget):
                 additional_data=add_data,
                 params=params,
                 stream_name=stream_name,
+                show_in_browser=show_in_browser,
+                web_port=web_port,
             )
             self.liveFits.append(livefit)
             if y_axes and y_axes[fit["y"]] == 2:
@@ -317,17 +421,28 @@ class PlotWidget(QWidget):
         label_n_data = QLabel("# data points:")
         self.lineEdit_n_data = QLineEdit(str(maxlen))
         self.lineEdit_n_data.returnPressed.connect(self.change_maxlen)
+        label_skip_n_points = QLabel("skip n points:")
+        self.lineEdit_skip_n_points = QLineEdit("0")
+        self.lineEdit_skip_n_points.returnPressed.connect(self.change_skip_n_points)
+        skip_tool_tip = "Skip n points in the plot before plotting the next value. This may be useful for long measurements to speed up plotting and free up memory."
+        label_skip_n_points.setToolTip(skip_tool_tip)
+        self.lineEdit_skip_n_points.setToolTip(skip_tool_tip)
         self.setLayout(QGridLayout())
-        self.layout().addWidget(self.plot_widget, 0, 1, 1, 4)
+        self.layout().addWidget(self.plot_widget, 0, 1, 1, 6)
         self.layout().addWidget(self.pushButton_show_options, 2, 1)
         self.layout().addWidget(self.pushButton_clear, 2, 2)
-        self.layout().addWidget(label_n_data, 2, 3)
-        self.layout().addWidget(self.lineEdit_n_data, 2, 4)
+        self.layout().addWidget(label_n_data, 2, 5)
+        self.layout().addWidget(self.lineEdit_n_data, 2, 6)
+        self.layout().addWidget(label_skip_n_points, 2, 3)
+        self.layout().addWidget(self.lineEdit_skip_n_points, 2, 4)
         self.layout().addWidget(self.plot_options, 0, 0, 3, 1)
         self.plot_options.hide()
-        self.setMinimumSize(500, 400)
-        self.adjustSize()
-        place_widget(self)
+        # self.setMinimumSize(500, 400)
+        if manual_plot_position:
+            place_widget(self, top_left_x, top_left_y, plot_width, plot_height)
+        else:
+            place_widget(self)
+        # self.adjustSize()
         self.change_maxlen()
 
     def show_again(self):
@@ -350,7 +465,7 @@ class PlotWidget(QWidget):
             if action.text() == "View All":
                 action.triggered.connect(self.auto_range)
             self.toolbar.addAction(action)
-        self.layout().addWidget(self.toolbar, 1, 1, 1, 4)
+        self.layout().addWidget(self.toolbar, 1, 1, 1, 6)
         self.plot_options.set_log()
 
     def auto_range(self):
@@ -373,6 +488,17 @@ class PlotWidget(QWidget):
                 return
         self.livePlot.change_maxlen(maxlen)
 
+    def change_skip_n_points(self):
+        text = self.lineEdit_skip_n_points.text()
+        if not text:
+            skip_n_points = 0
+        else:
+            try:
+                skip_n_points = int(text)
+            except ValueError:
+                return
+        self.livePlot.change_skip_n_points(skip_n_points)
+
     def show_options(self):
         """
         Shows or hides the plot options. If the options are shown, the button text is changed to "Hide Options", otherwise it is changed to "Show Options".
@@ -498,7 +624,7 @@ class Plot_Options(Ui_Plot_Options, QWidget):
 
             color = item.opts["pen"].color()
             colorwidge = QPushButton(color.name())
-            colorwidge.clicked.connect(lambda n=i: self.change_color(n))
+            colorwidge.clicked.connect(lambda state, n=i: self.change_color(n))
             self.tableWidget.setCellWidget(i, 3, colorwidge)
 
     def change_symbol(self, symbol, row):
@@ -660,7 +786,7 @@ class LivePlot(QObject, CallbackBase):
             self._epoch = epoch
             self.setup_is_done = True
 
-        self.x_data = []
+        self.x_data = ListDeque_skip(maxlen=maxlen)
         self.y_data = {}
         self.y_axes = y_axes or {}
         self.maxlen = maxlen
@@ -669,7 +795,7 @@ class LivePlot(QObject, CallbackBase):
         self.y_names = y_names
         self.ys = get_obj_fields(y_names)
         for y in y_names:
-            self.y_data[y] = []
+            self.y_data[y] = ListDeque_skip(maxlen=maxlen)
         self.current_plots = {}
         self.legend = None
         self.desc = []
@@ -798,6 +924,10 @@ class LivePlot(QObject, CallbackBase):
             if doc["descriptor"] in self.descs_fit_readying:
                 self.descs_fit_readying[doc["descriptor"]].get_ready()
             return
+        # Check to see if doc["data"] contains keys matching any on the self.ys strings or self.x
+        if not (any(key in s for key in doc["data"] for s in self.ys)): #or self.x in doc["data"]):
+            print("The data of the event does not match the data ")
+            return
         try:
             new_x = doc["data"][self.x]
         except KeyError:
@@ -817,7 +947,11 @@ class LivePlot(QObject, CallbackBase):
             except KeyError:
                 if not self.eva.is_to_date(doc["time"]):
                     self.eva.event(doc)
-                new_y[y] = self.eva.eval(y, do_not_reraise=True)
+                try:
+                    new_y[y] = self.eva.eval(y, do_not_reraise=True)
+                except ValueError as e:
+                    print("Error getting data for plot", e)
+                    return
         self.update_caches(new_x, new_y)
         for fit in self.fitPlots:
             fit.event(doc)
@@ -875,23 +1009,23 @@ class LivePlot(QObject, CallbackBase):
     def clear_plot(self):
         for y in self.current_plots:
             self.current_plots[y].setData([], [])
-        self.x_data = []
+        self.x_data = ListDeque_skip(maxlen=self.maxlen)
         for y in self.y_data:
-            self.y_data[y] = []
+            self.y_data[y] = ListDeque_skip(maxlen=self.maxlen)
         for fit in self.fitPlots:
             fit.clear_plot()
         self.update_plot()
 
     def change_maxlen(self, maxlen):
         self.maxlen = maxlen
-        if maxlen < np.inf:
-            self.x_data = deque(self.x_data, maxlen=maxlen)
-            for y in self.y_data:
-                self.y_data[y] = deque(self.y_data[y], maxlen=maxlen)
-        else:
-            self.x_data = list(self.x_data)
-            for y in self.y_data:
-                self.y_data[y] = list(self.y_data[y])
+        self.x_data.change_maxlen(maxlen)
+        for y in self.y_data:
+            self.y_data[y].change_maxlen(maxlen)
+
+    def change_skip_n_points(self, n):
+        self.x_data.skip_n_points = n
+        for y in self.y_data:
+            self.y_data[y].skip_n_points = n
 
 
 class LiveFitPlot(CallbackBase):
@@ -979,7 +1113,9 @@ class LiveFitPlot(CallbackBase):
     def fit_has_result(self):
         if self.livefit.result is not None:
             x_data = self.livefit.independent_vars_data[self.__x_key]
-            x_points = np.linspace(np.min(x_data), np.max(x_data), self.num_points)
+            x_points = np.linspace(
+                np.min(x_data), np.max(x_data), max(self.num_points, len(x_data))
+            )
             kwargs = {self.__x_key: x_points}
             kwargs.update(self.livefit.result.values)
             self.y_data = self.livefit.result.model.eval(**kwargs)
@@ -1053,13 +1189,18 @@ class PlotWidget_2D(QWidget):
         y_name,
         z_name,
         parent=None,
-        namespace=None,
+        evaluator=None,
         xlabel="",
         ylabel="",
         zlabel="",
         title="",
         maxlen=np.inf,
         stream_name="primary",
+        manual_plot_position=False,
+        top_left_x="",
+        top_left_y="",
+        plot_width="",
+        plot_height="",
         **kwargs,
     ):
         super().__init__(parent=parent)
@@ -1077,7 +1218,7 @@ class PlotWidget_2D(QWidget):
         self.stream_name = stream_name
 
         self.toolbar = None
-        eva = Evaluator(namespace=namespace)
+        eva = evaluator
         self.livePlot = LivePlot_2D(
             x_name,
             y_name,
@@ -1102,8 +1243,11 @@ class PlotWidget_2D(QWidget):
         self.layout().addWidget(label_n_data, 2, 1)
         self.layout().addWidget(self.lineEdit_n_data, 2, 2)
         self.make_toolbar()
+        if manual_plot_position:
+            place_widget(self, top_left_x, top_left_y, plot_width, plot_height)
+        else:
+            place_widget(self)
         self.adjustSize()
-        place_widget(self)
         self.change_maxlen()
 
     def change_maxlen(self):
