@@ -6,6 +6,7 @@ import lmfit
 from collections import deque
 
 import matplotlib
+import requests
 
 matplotlib.use("Qt5Agg")
 import matplotlib.pyplot as plt
@@ -409,10 +410,14 @@ class LiveFit_Eva(LiveFit):
         name="",
         params=None,
         stream_name="primary",
+        show_in_browser=False,
+        web_port=None,
     ):
         super().__init__(
             model=model, y=y, independent_vars=independent_vars, init_guess=init_guess
         )
+        self.web_port = web_port
+        self.show_in_browser = show_in_browser
         self.eva = evaluator
         self.name = f"{name}_{stream_name}"
         self.params = params
@@ -519,6 +524,13 @@ class LiveFit_Eva(LiveFit):
         for d in self.additional_data:
             self.additional_data[d].append(self.eva.eval(d))
         self.__stale = False
+        if self.show_in_browser:
+            try:
+                request = requests.post(f"http://127.0.0.1:{self.web_port}/fit_result", json={"best_fit": self.result.best_fit.tolist(), "name": self.name, "fit_params": self.result.best_values, "model_name":self.model.name, "y_axis_name":self.y})
+                if request.status_code != 200:
+                    print(f"Error: {request.status_code}")
+            except requests.exceptions.ConnectionError:
+                print("ConnectionError. Failed to send fit result to browser.")
         # self.ophyd_fit.update_data(self.result, self.timestamp)
         self.parent_plot.fit_has_result()
 
@@ -1642,25 +1654,4 @@ class MultiPlot_NoBluesky(QObject):
         """ """
         self.xdata.clear()
         for y in self.ydata:
-            self.ydata[y].clear()
-
-
-# if __name__ == '__main__':
-#     from bluesky import RunEngine
-#     from bluesky.plans import scan
-#     from ophyd.sim import motor, det
-#
-#     motor.delay = 0.1
-#
-#     def plan():
-#         for i in range(4, 5):
-#             yield from scan([det], motor, -5, 5, 3**i)
-#
-#     RE = RunEngine()
-#     app = QApplication(sys.argv)
-#     # myapp = PlotWidget(run_engine=RE, x_name='motor', y_names=['det'], title='test', xlabel='aaaa', ylabel='bbbb')
-#     myapp = PlotWidget('motor', ['det', 'det**2', 'sin(motor)'])
-#     myapp.show()
-#     RE.subscribe(myapp.livePlot)
-#     RE(plan())
-#     sys.exit(app.exec())
+            self.ydata[y].clear()
\ No newline at end of file
