@@ -45,14 +45,8 @@ class Measurement_Protocol:
         Dictionary of the channel-names and channels used inside the protocol
     name : str
         The name of the protocol. This also appears in the main UI
-    channel_metadata : dict
-        Currently deprecated. Only used for specific nexus output.
-    config_metadata : dict
-        Currently deprecated. Only used for specific nexus output.
-    metadata : dict
-        Currently deprecated. Only used for specific nexus output.
     use_nexus : bool
-        Currently deprecated. Only used for specific nexus output.
+        Whether to write a NeXus-entry into the datafile as well.
     """
 
     def __init__(
@@ -61,10 +55,7 @@ class Measurement_Protocol:
         plots=None,
         channels=None,
         name="",
-        channel_metadata=None,
-        metadata=None,
-        use_nexus=False,
-        config_metadata=None,
+        use_nexus=True,
         **kwargs,
     ):
         if plots is None:
@@ -73,10 +64,6 @@ class Measurement_Protocol:
             loop_steps = []
         if channels is None:
             channels = {}
-        if channel_metadata is None:
-            channel_metadata = {}
-        if metadata is None:
-            metadata = {}
         self.description = kwargs["description"] if "description" in kwargs else ""
         self.export_csv = kwargs["export_csv"] if "export_csv" in kwargs else False
         self.export_json = kwargs["export_json"] if "export_json" in kwargs else False
@@ -89,6 +76,14 @@ class Measurement_Protocol:
             kwargs["use_end_protocol"] if "use_end_protocol" in kwargs else False
         )
         self.end_protocol = kwargs["end_protocol"] if "end_protocol" in kwargs else ""
+        self.live_variable_update = (
+            kwargs["live_variable_update"]
+            if "live_variable_update" in kwargs
+            else False
+        )
+        self.allow_live_comments = (
+            kwargs["allow_live_comments"] if "allow_live_comments" in kwargs else False
+        )
         self.loop_steps = loop_steps
         self.loop_step_dict = {}
         for step in self.loop_steps:
@@ -99,10 +94,9 @@ class Measurement_Protocol:
         self.loop_step_variables = {}
         self.channels = channels
         self.name = name or "Protocol"
-        self.channel_metadata = channel_metadata
-        self.config_metadata = config_metadata
-        self.metadata = metadata
         self.use_nexus = use_nexus
+        self.measurement_description = ""
+        self.tags = []
 
     def update_variables(self):
         """Update all the variables provided by loopsteps."""
@@ -112,32 +106,6 @@ class Measurement_Protocol:
                 continue
             step.update_variables()
 
-    def get_nexus_paths(self):
-        """Get a dictionary containing the paths used for the output
-        NeXus-file. Currently deprecated."""
-        paths = {}
-        for i, name in enumerate(self.metadata["Name"]):
-            paths[self.metadata["NeXus-path"][i]] = f"metadata_start/{name}"
-        for i, name in enumerate(self.channel_metadata["Channel"]):
-            paths[self.channel_metadata["NeXus-path"][i]] = f"data/{name}"
-        for i, name in enumerate(self.config_metadata["Configuration"]):
-            path = ""
-            for dev in variables_handling.devices:
-                if dev in name:
-                    rn = name.split(dev)[1][1:]
-                    device = variables_handling.devices[dev]
-                    if (
-                        rn in device.get_config()
-                        or rn in device.get_config()
-                        or rn in device.get_passive_config()
-                    ):
-                        path = f"metadata_start/device_config/{dev}/{name}"
-                        break
-            if not path:
-                raise Exception(f"Cannot find {name} in any configuration!")
-            paths[self.config_metadata["NeXus-path"][i]] = path
-        return paths
-
     def add_loop_step(self, loop_step, position=-1, parent_step_name=None, model=None):
         """Adds a loop_step to the protocol (or the parent_step)at the specified
         position. Also appends the loop_step to the given model. The
@@ -304,7 +272,7 @@ class Measurement_Protocol:
     def get_plan_string(self):
         """Get the string for the protocol-plan, including the loopsteps."""
         variables_handling.current_protocol = self
-        plan_string = f'\n\n\ndef {self.name.replace(" ","_")}_plan_inner(devs, eva=None, stream_name="primary", runEngine=None):\n'
+        plan_string = f'\n\n\ndef {self.name.replace(" ","_")}_plan_inner(devs, stream_name="primary", runEngine=None):\n'
         prot_vars = dict(variables_handling.protocol_variables)
         if "StartTime" in prot_vars:
             prot_vars.pop("StartTime")
@@ -321,18 +289,33 @@ class Measurement_Protocol:
                 continue
             plan_string += step.get_protocol_string(n_tabs=1)
         plan_string += f'\n\n\ndef {self.name.replace(" ","_")}_plan(devs, md=None, runEngine=None, stream_name="primary"):\n'
-        plan_string += "\teva = Evaluator(namespace=namespace)\n"
         plan_string += "\tsub_eva = runEngine.subscribe(eva)\n"
         plan_string += "\tyield from bps.open_run(md=md)\n"
         if self.use_end_protocol:
             plan_string += "\ttry:\n"
-            plan_string += f'\t\tyield from {self.name.replace(" ", "_")}_plan_inner(devs, eva, stream_name, runEngine)\n'
+            plan_string += f'\t\tyield from {self.name.replace(" ", "_")}_plan_inner(devs, stream_name, runEngine)\n'
             plan_string += "\t\tyield from helper_functions.get_fit_results(all_fits, namespace, True)\n"
             plan_string += "\tfinally:\n"
             plan_string += "\t\tyield from ending_steps(runEngine, devs)\n"
         else:
-            plan_string += f'\tyield from {self.name.replace(" ", "_")}_plan_inner(devs, eva, stream_name, runEngine)\n'
+            plan_string += f'\tyield from {self.name.replace(" ", "_")}_plan_inner(devs, stream_name, runEngine)\n'
             plan_string += "\tyield from helper_functions.get_fit_results(all_fits, namespace, True)\n"
+        check_live_windows = False
+        if self.allow_live_comments:
+            check_live_windows = True
+        if check_live_windows:
+            plan_string += "\tfinished = False\n"
+            plan_string += "\twhile not finished:\n"
+            plan_string += "\t\tfinished = True\n"
+            plan_string += "\t\tfor window in live_windows:\n"
+            plan_string += "\t\t\tif hasattr(window, '_is_finished') and not window._is_finished:\n"
+            plan_string += "\t\t\t\tfinished = False\n"
+            plan_string += "\tlive_metadata = {}\n"
+            plan_string += "\tfor window in live_windows:\n"
+            plan_string += "\t\tif hasattr(window, 'get_metadata'):\n"
+            plan_string += "\t\t\tlive_metadata.update(window.get_metadata())\n"
+            plan_string += "\tlive_metadata_signal = variable_reading.Variable_Signal(name='live_metadata', variables_dict=live_metadata)\n"
+            plan_string += '\tyield from bps.trigger_and_read([live_metadata_signal], name="_live_metadata_reading_")\n'
         plan_string += "\tyield from bps.close_run()\n"
         plan_string += "\trunEngine.unsubscribe(sub_eva)\n"
         return plan_string
@@ -366,6 +349,30 @@ class Measurement_Protocol:
         add_main_string += "\treturn returner\n\n\n"
         return add_main_string
 
+    def get_live_interaction_string(self):
+        """Returns the string for the live interaction of the protocol."""
+        live_string = "def create_live_windows():\n"
+        live_string += "\tglobal live_windows\n"
+        if self.live_variable_update:
+            live_string += (
+                "\tfrom nomad_camels.ui_widgets.variable_table import VariableBox\n"
+            )
+            live_string += f"\tvariables = {self.variables}\n"
+            live_string += f'\tvariable_box = VariableBox(editable_names=False, variables=variables, name="{self.name}")\n'
+            live_string += "\n\tdef update_variables(new_variables):\n"
+            live_string += "\t\tglobal namespace\n"
+            live_string += "\t\tnamespace.update(new_variables)\n\n"
+            live_string += (
+                "\tvariable_box.new_values_signal.connect(update_variables)\n"
+            )
+            live_string += "\tlive_windows.append(variable_box)\n"
+            live_string += "\tvariable_box.show()\n"
+        if self.allow_live_comments:
+            live_string += "\tcommenting_box = helper_functions.Commenting_Box()\n"
+            live_string += "\tlive_windows.append(commenting_box)\n"
+        live_string += "\treturn live_windows\n\n\n"
+        return live_string
+
     def get_total_steps(self):
         """Returns the total number of steps (including repetitions for loops)"""
         total = 0
@@ -500,51 +507,16 @@ class General_Protocol_Settings(Ui_Protocol_Settings, QWidget):
             file_extension="*.cprot",
         )
         self.checkBox_perform_at_end.setToolTip(
-            "Select a protocol to be performed at the end of this protocol or when it is aborted by the user.\nThis may be useful e.g. to turn something of in a controlled way.\nThis is NOT executed, when the protocol is run as a subprotocol."
+            "Select a protocol to always be performed at the end of this protocol, no matter whether it runs smoothly or fails or is stopped by the user.\nThis may be useful e.g. to turn something off in a controlled way.\nThis is NOT executed, when the protocol is run as a subprotocol."
         )
         self.ending_protocol_selection.setToolTip(
             "Select a protocol to be performed at the end of this protocol or when it is aborted by the user.\nThis may be useful e.g. to turn something of in a controlled way.\nThis is NOT executed, when the protocol is run as a subprotocol."
         )
-        self.checkBox_perform_at_end.stateChanged.connect(
-            self.check_use_ending_steps
-        )
+        self.checkBox_perform_at_end.stateChanged.connect(self.check_use_ending_steps)
         self.check_use_ending_steps()
 
         self.plot_widge = Plot_Button_Overview(self, self.protocol.plots)
 
-        cols = ["Channel", "NeXus-path"]
-        comboBoxes = {"Channel": list(variables_handling.channels.keys())}
-        self.table_channel_NX_paths = AddRemoveTable(
-            headerLabels=cols,
-            title="Channel-NeXus-Path",
-            comboBoxes=comboBoxes,
-            tableData=self.protocol.channel_metadata,
-        )
-
-        cols = ["Configuration", "NeXus-path"]
-        configs = []
-        for dev in variables_handling.devices:
-            device = variables_handling.devices[dev]
-            allconf = []
-            allconf += list(device.get_passive_config().keys())
-            allconf += list(device.get_config().keys())
-            for key in allconf:
-                configs.append(f"{device.name}_{key}")
-        comboBoxes = {"Configuration": configs}
-        self.table_config_NX_paths = AddRemoveTable(
-            headerLabels=cols,
-            title="Config-NeXus-Path",
-            comboBoxes=comboBoxes,
-            tableData=self.protocol.config_metadata,
-        )
-
-        cols = ["Name", "NeXus-path", "Value"]
-        self.table_metadata = AddRemoveTable(
-            headerLabels=cols, title="NeXus-Metadata", tableData=self.protocol.metadata
-        )
-
-        self.checkBox_NeXus = QCheckBox("Use NeXus-output")
-        self.checkBox_NeXus.clicked.connect(self.enable_nexus)
         self.checkBox_NeXus.setChecked(self.protocol.use_nexus)
         self.lineEdit_protocol_name.textChanged.connect(self.name_change)
         self.name_change()
@@ -560,8 +532,13 @@ class General_Protocol_Settings(Ui_Protocol_Settings, QWidget):
         self.checkBox_no_config.clicked.connect(self.enable_disable_config)
         self.adjust_text_edit_size_prot()
 
-        self.radioButton_h5_during.setChecked(self.protocol.h5_during_run)
-        self.radioButton_h5_after.setChecked(not self.protocol.h5_during_run)
+        self.checkBox_live_variables.setChecked(self.protocol.live_variable_update)
+        self.checkBox_live_comments.setChecked(self.protocol.allow_live_comments)
+
+        if self.protocol.h5_during_run:
+            self.comboBox_h5.setCurrentIndex(0)
+        else:
+            self.comboBox_h5.setCurrentIndex(1)
 
         self.layout().addWidget(self.textEdit_desc_protocol, 5, 0, 1, 6)
 
@@ -570,8 +547,6 @@ class General_Protocol_Settings(Ui_Protocol_Settings, QWidget):
         self.layout().addWidget(self.ending_protocol_selection, 21, 0, 1, 6)
         # ! Ui_Protocol_Settings.ui file adds the Variables Table at position 8 and 9 !!!
 
-        self.checkBox_NeXus.setHidden(True)
-        self.enable_nexus()
         self.update_variable_select()
         self.variable_table.selectionModel().selectionChanged.connect(
             self.update_variable_select
@@ -611,44 +586,6 @@ class General_Protocol_Settings(Ui_Protocol_Settings, QWidget):
             if result != QMessageBox.Yes:
                 self.checkBox_no_config.setChecked(False)
 
-    def enable_nexus(self):
-        """When the checkBox_NeXus is clicked, enables / disables the
-        other widgets for the nexus-definition.
-
-        Parameters
-        ----------
-
-        Returns
-        -------
-
-        """
-        nx = self.checkBox_NeXus.isChecked()
-        self.table_channel_NX_paths.setHidden(not nx)
-        self.table_metadata.setHidden(not nx)
-        self.table_config_NX_paths.setHidden(not nx)
-
-    # def get_unique_name(self, name="name"):
-    #     """Checks whether name already exists in the variables of the
-    #     protocol and returns a unique name (with added _i).
-
-    #     Parameters
-    #     ----------
-    #     name :
-    #          (Default value = 'name')
-
-    #     Returns
-    #     -------
-
-    #     """
-    #     i = 1
-    #     while name in self.protocol.variables:
-    #         if "_" not in name:
-    #             name += f"_{i}"
-    #         else:
-    #             name = f'{name.split("_")[0]}_{i}'
-    #         i += 1
-    #     return name
-
     def add_variable(self):
         """Add a variable to the list, given a unique name, then updates
         the protocol.
@@ -701,17 +638,16 @@ class General_Protocol_Settings(Ui_Protocol_Settings, QWidget):
         self.protocol.name = self.lineEdit_protocol_name.text()
         self.protocol.description = self.textEdit_desc_protocol.toPlainText()
         self.protocol.plots = self.plot_widge.plot_data
-        self.protocol.metadata = self.table_metadata.update_table_data()
-        self.protocol.channel_metadata = self.table_channel_NX_paths.update_table_data()
-        self.protocol.config_metadata = self.table_config_NX_paths.update_table_data()
         self.protocol.export_csv = self.checkBox_csv_exp.isChecked()
         self.protocol.export_json = self.checkBox_json_exp.isChecked()
         self.protocol.skip_config = self.checkBox_no_config.isChecked()
         self.variable_table.update_variables()
         self.protocol.use_nexus = self.checkBox_NeXus.isChecked()
-        self.protocol.h5_during_run = self.radioButton_h5_during.isChecked()
+        self.protocol.h5_during_run = self.comboBox_h5.currentIndex() == 0
         self.protocol.use_end_protocol = self.checkBox_perform_at_end.isChecked()
         self.protocol.end_protocol = self.ending_protocol_selection.get_path()
+        self.protocol.live_variable_update = self.checkBox_live_variables.isChecked()
+        self.protocol.allow_live_comments = self.checkBox_live_comments.isChecked()
 
     # def load_variables(self):
     #     """Called when starting, loads the variables from the protocol
